import {
  Resolver,
  Query,
  Mutation,
  Args,
  Int,
  ResolveField,
  Parent,
} from '@nestjs/graphql';
import { ArticlesService } from './articles.service';
import { Article } from './entities/article.entity';
import { CreateArticleInput } from './dto/create-article.input';
import { UpdateArticleInput } from './dto/update-article.input';
import * as marked from 'marked';
import { getLanguage, highlight } from 'highlight.js';

@Resolver(() => Article)
export class ArticlesResolver {
  constructor(private readonly articlesService: ArticlesService) {}

  @Mutation(() => Article)
  createArticle(
    @Args('createArticleInput') createArticleInput: CreateArticleInput,
  ) {
    return this.articlesService.create(createArticleInput);
  }

  @Query(() => [Article], { name: 'articles' })
  async findAll() {
    return await this.articlesService.findAll();
  }

  @Query(() => Article, { name: 'article' })
  findOne(@Args('id', { type: () => String }) id: string) {
    return this.articlesService.findOne(id);
  }

  @Mutation(() => Article)
  async updateArticle(
    @Args('updateArticleInput') updateArticleInput: UpdateArticleInput,
  ) {
    const article = await this.articlesService.findOne(updateArticleInput.id);
    return this.articlesService.update(article, updateArticleInput);
  }

  @Mutation(() => Int)
  removeArticle(@Args('id', { type: () => String }) id: string) {
    return this.articlesService.remove(id);
  }

  @ResolveField(() => String)
  async html(@Parent() article: Article) {
    const tokens = marked.lexer(article.content);
    const index = tokens.findIndex((token) => ['heading'].includes(token.type));
    if (index !== -1) {
      tokens.splice(index, 1);
    }
    return marked.parser(tokens, {
      gfm: true,
      smartLists: true,
      smartypants: true,
      langPrefix: 'hljs language-',
      highlight: (code, language) => {
        return highlight(code, {
          language: getLanguage(language) ? language : 'plaintext',
        }).value;
      },
    });
  }

  @ResolveField(() => String, { nullable: true })
  async description(@Parent() article: Article) {
    const tokens = marked.lexer(article.content);
    const token = tokens.find((token) =>
      ['blockquote', 'paragraph'].includes(token.type),
    ) as { text: string };

    return token?.text;
  }
}
